@@ -1,4 +1,4 @@
-/*	$OpenBSD: rijndael.h,v 1.14 2014/04/29 15:42:07 markus Exp $ */
+/*	$OpenBSD: rijndael.h,v 1.15 2021/09/28 11:14:50 dtucker Exp $ */
 
 /**
  * rijndael-alg-fst.h
@@ -38,8 +38,7 @@ typedef unsigned short	u16;
 typedef unsigned int	u32;
 
 int	rijndaelKeySetupEnc(unsigned int [], const unsigned char [], int);
-void	rijndaelEncrypt(const unsigned int [], int, const unsigned char [],
-	    unsigned char []);
+void	rijndaelEncrypt(const unsigned int [], int, const u8 [16], u8 [16]);
 
 /*  The structure for key information */
 typedef struct {
